<?php

namespace jrosset\CliProgram;

use jrosset\CliProgram\CommandCall\CommandCall;
use jrosset\CliProgram\CommandCall\CommandCallsList;
use ReflectionClass;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Output\OutputInterface;
use Throwable;

/**
 * A basic command
 */
class BaseCommand extends Command {
    /**
     * @return string|null The command default name
     */
    public static function getDefaultName (): ?string {
        $reflectionClass = new ReflectionClass(static::class);
        if ($attribute = $reflectionClass->getAttributes(CliCommand::class)) {
            /** @var CliCommand $cliCommandAttribute */
            $cliCommandAttribute = $attribute[0]->newInstance();
            if ($cliCommandAttribute->name !== null) {
                return $cliCommandAttribute->name;
            }
        }
        if (($name = parent::getDefaultName()) !== null) {
            return $name;
        }

        $classShortName = $reflectionClass->getShortName();
        return mb_strtolower(mb_substr($classShortName, 0, 1)) . mb_substr($classShortName, 1);
    }
    /**
     * @return string[] The command default aliases
     */
    public static function getDefaultAliases (): array {
        $reflectionClass = new ReflectionClass(static::class);
        if ($attribute = $reflectionClass->getAttributes(CliCommand::class)) {
            /** @var CliCommand $cliCommandAttribute */
            $cliCommandAttribute = $attribute[0]->newInstance();
            return $cliCommandAttribute->aliases;
        }
        return [];
    }
    /**
     * @return string|null The command default description
     */
    public static function getDefaultDescription (): ?string {
        if ($attribute = (new ReflectionClass(static::class))->getAttributes(CliCommand::class)) {
            /** @var CliCommand $cliCommandAttribute */
            $cliCommandAttribute = $attribute[0]->newInstance();
            return $cliCommandAttribute->description;
        }
        return parent::getDefaultDescription();
    }
    /**
     * @return bool Is the command hidden from command list by default ?
     */
    public static function getDefaultHidden (): bool {
        if ($attribute = (new ReflectionClass(static::class))->getAttributes(CliCommand::class)) {
            /** @var CliCommand $cliCommandAttribute */
            $cliCommandAttribute = $attribute[0]->newInstance();
            return $cliCommandAttribute->hidden;
        }
        return false;
    }

    /**
     * Initialize the command
     *
     * @param string|null          $name        The command name ; Null if the default one (cf. {@see static::getDefaultName()})
     * @param string[]|string|null $aliases     The command aliases ; Null if the default one (cf. {@see static::getDefaultAliases()})
     * @param string|null          $description The command description ; Null if the default one (cf. {@see static::getDefaultDescription()})
     * @param bool|null            $hidden      Is the command hidden from command list ? Null if the default one (cf. {@see static::getDefaultHidden()})
     */
    public function __construct (?string $name = null, array|string|null $aliases = null, ?string $description = null, ?bool $hidden = null) {
        parent::__construct($name);
        $this->setAliases(is_string($aliases) ? [$aliases] : $aliases ?? static::getDefaultAliases());
        $this->setDescription($description ?? $this->getDescription());
        $this->setHidden($hidden ?? static::getDefaultHidden());
    }

    /**
     * Run a list of subcommands
     *
     * Stop à the first subcommand returning other than {@see Command::SUCCESS}
     *
     * @param CommandCallsList|CommandCall $subcommandList The subcommands to run
     * @param OutputInterface              $output         The output
     *
     * @return int The last executed subcommand return code
     *
     * @throws Throwable If an error occurs
     */
    public function runSubCommands (CommandCallsList|CommandCall $subcommandList, OutputInterface $output): int {
        if (!$subcommandList instanceof CommandCallsList) {
            $subcommandList = new CommandCallsList([$subcommandList]);
        }

        /** @var CommandCall $subcommand */
        foreach ($subcommandList as $subcommand) {
            if (($returnCode = $subcommand->run($this->getApplication(), $output)) !== Command::SUCCESS) {
                return $returnCode;
            }
        }

        return Command::SUCCESS;
    }
}