<?php

namespace jrosset\CliProgram\Validation\Validators;

use InvalidArgumentException;
use Stringable;

/**
 * An argument/option value validator for a directory path
 */
class DirectoryValidator implements IValidator {
    use TInternalValueValidator;

    /**
     * @var FilesystemValidationOptionsList The options
     */
    private FilesystemValidationOptionsList $options;

    /**
     * Initialization
     *
     * @param FilesystemValidationOptionsList|FilesystemValidationOption[]|FilesystemValidationOption|null $options The options
     */
    public function __construct (FilesystemValidationOptionsList|array|FilesystemValidationOption|null $options = null) {
        $this->setOptions($options);
    }

    /**
     * Get a valid default value from the initial/given default value
     *
     * @param string|Stringable|null $default The initial/given default value
     *
     * @return string|bool|int|float|array|null The valid default value
     *
     * @throws InvalidArgumentException If the default value is not a string or null
     * @throws InvalidArgumentException If the default directory doesn't match the options
     */
    public function getValidDefault (mixed $default): string|bool|int|float|array|null {
        if ($default === null) {
            return null;
        }
        if (!is_string($default) && !$default instanceof Stringable) {
            throw new InvalidArgumentException('The default value must be a string or null');
        }

        if ($this->getOptions()->contains(FilesystemValidationOption::IsReadable) && !is_readable($default)) {
            throw new InvalidArgumentException('The default value is not readable');
        }
        if ($this->getOptions()->contains(FilesystemValidationOption::IsWritable) && !is_writable($default)) {
            throw new InvalidArgumentException('The default value is not writable');
        }
        if ($this->getOptions()->contains(FilesystemValidationOption::MustExists) && !file_exists($default)) {
            throw new InvalidArgumentException('The default value doesn\'t exist');
        }
        return $default;
    }

    /**
     * @inheritDoc
     */
    public function validate (mixed $value): bool {
        if (!is_string($value) && !$value instanceof Stringable) {
            return false;
        }

        if ($this->getOptions()->contains(FilesystemValidationOption::IsReadable->name) && !is_readable($value)) {
            return false;
        }
        if ($this->getOptions()->contains(FilesystemValidationOption::IsWritable->name) && !is_writable($value)) {
            return false;
        }
        if ($this->getOptions()->contains(FilesystemValidationOption::MustExists->name) && !file_exists($value)) {
            return false;
        }

        $this->setValue($value);
        return true;
    }

    /**
     * The options
     *
     * @return FilesystemValidationOptionsList The options
     */
    public function getOptions (): FilesystemValidationOptionsList {
        return $this->options;
    }
    /**
     * Replace the options
     *
     * @param FilesystemValidationOptionsList|FilesystemValidationOption[]|FilesystemValidationOption|null $options The options
     *
     * @return $this
     */
    public function setOptions (FilesystemValidationOptionsList|array|FilesystemValidationOption|null $options = null): static {
        $this->options = match (true) {
            $options instanceof FilesystemValidationOptionsList => $options,
            $options instanceof FilesystemValidationOption      => new FilesystemValidationOptionsList([$options]),
            is_array($options)                                  => new FilesystemValidationOptionsList($options),
            $options === null                                   => new FilesystemValidationOptionsList(),
        };
        return $this;
    }
}