<?php

namespace jrosset\CliProgram\Validation\Validators;

use InvalidArgumentException;

/**
 * An argument/option value validator for a file path
 */
class FileValidator extends DirectoryValidator {
    /**
     * @var string|null The allowed extensions (regex pattern)
     */
    private ?string $extensionsPattern;

    /**
     * Initialization
     *
     * @param string|null                                                                                  $extensionsPattern The allowed extensions (regex pattern)
     * @param FilesystemValidationOptionsList|FilesystemValidationOption[]|FilesystemValidationOption|null $options           The options
     */
    public function __construct (?string $extensionsPattern = null, FilesystemValidationOptionsList|FilesystemValidationOption|array|null $options = null) {
        parent::__construct($options);
        $this->setExtensionsPattern($extensionsPattern);
    }

    /**
     * @inheritDoc
     */
    public function getValidDefault (mixed $default): string|bool|int|float|array|null {
        if (($default = parent::getValidDefault($default)) === null) {
            return null;
        }
        if ($this->extensionsPattern !== null && preg_match($this->extensionsPattern, $default) !== 1) {
            throw new InvalidArgumentException('The default value has not a valid extension');
        }
        return $default;
    }
    /**
     * @inheritDoc
     */
    public function validate (mixed $value): bool {
        if (!parent::validate($value)) {
            return false;
        }
        if ($this->extensionsPattern !== null && preg_match($this->extensionsPattern, $value) !== 1) {
            return false;
        }
        return true;
    }

    /**
     * The allowed extensions (regex pattern)
     *
     * @return string|null The allowed extensions (regex pattern)
     */
    public function getExtensionsPattern (): ?string {
        return $this->extensionsPattern;
    }
    /**
     * Replace the allowed extensions
     *
     * @param string|null $extensionsPattern The allowed extensions (regex pattern)
     *
     * @return $this
     */
    public function setExtensionsPattern (?string $extensionsPattern = null): static {
        $this->extensionsPattern = $extensionsPattern;
        return $this;
    }
}