<?php

namespace jrosset\CliProgram\Validation\Validators;

/**
 * An argument/option value validator based on a regex
 */
class RegexValidator implements IValidator {
    use TIdenticalValidDefaultValidator;

    public const GROUP_VALUE = 'value';

    /**
     * @var string The regex pattern
     */
    private string $pattern;
    /**
     * @var string[] The capturing groups
     */
    private array $matches;

    /**
     * Create a validator
     *
     * @param string $pattern The regex pattern
     */
    public function __construct (string $pattern) {
        $this->pattern = $pattern;
        $this->matches = [];
    }

    /**
     * @inheritDoc
     */
    public function validate (mixed $value): bool {
        return preg_match($this->pattern, $value, $this->matches) === 1;
    }

    /**
     * The regex pattern
     *
     * @return string The regex pattern
     */
    public function getPattern (): string {
        return $this->pattern;
    }
    /**
     * Set the regex pattern
     *
     * @param string $pattern The regex pattern
     *
     * @return $this
     */
    public function setPattern (string $pattern): self {
        $this->pattern = $pattern;
        return $this;
    }

    /**
     * The capturing groups
     *
     * @return string[] The capturing groups
     */
    public function getMatches (): array {
        return $this->matches;
    }
    /**
     * A capturing group
     *
     * @param int|string $group The group name/index
     *
     * @return string|null The capturing group or Null if not set
     */
    public function getMatch (int|string $group): ?string {
        return $this->getMatches()[$group] ?? null;
    }
    /**
     * Has a capturing group ?
     *
     * @param int|string $group The group name/index
     *
     * @return bool True if the capturing group exists and is not null
     */
    public function hasMatch (int|string $group): bool {
        return $this->getMatches()[$group] !== null;
    }

    /**
     * @inheritDoc
     */
    public function getValue (): ?string {
        return $this->getMatch(static::GROUP_VALUE) ?? $this->getMatch(0);
    }
}